@@ -845,6 +845,8 @@ reload_wm_class (MetaWindow    *window,
 
       if (value->v.class_hint.res_class)
         window->res_class = g_strdup (value->v.class_hint.res_class);
+
+      g_object_notify (G_OBJECT (window), "wm-class");
     }
 
   meta_verbose ("Window %s class: '%s' name: '%s'\n",
@@ -1466,21 +1468,49 @@ reload_transient_for (MetaWindow    *window,
                       MetaPropValue *value,
                       gboolean       initial)
 {
-  window->xtransient_for = None;
-  
-  if (value->type != META_PROP_VALUE_INVALID)
-    window->xtransient_for = value->v.xwindow;
+  MetaWindow *parent = NULL;
+  Window transient_for, old_transient_for;
 
-  /* Make sure transient_for is valid */
-  if (window->xtransient_for != None &&
-      meta_display_lookup_x_window (window->display, 
-                                    window->xtransient_for) == NULL)
+  if (value->type != META_PROP_VALUE_INVALID)
     {
-      meta_warning (_("Invalid WM_TRANSIENT_FOR window 0x%lx specified "
-                      "for %s.\n"),
-                    window->xtransient_for, window->desc);
-      window->xtransient_for = None;
+      transient_for = value->v.xwindow;
+
+      parent = meta_display_lookup_x_window (window->display, transient_for);
+      if (!parent)
+        {
+          meta_warning (_("Invalid WM_TRANSIENT_FOR window 0x%lx specified "
+                          "for %s.\n"),
+                        transient_for, window->desc);
+          transient_for = None;
+        }
+
+      /* Make sure there is not a loop */
+      while (parent)
+        {
+          if (parent == window)
+            {
+              meta_warning (_("WM_TRANSIENT_FOR window 0x%lx for %s "
+                              "would create loop.\n"),
+                            transient_for, window->desc);
+              transient_for = None;
+              break;
+            }
+
+          parent = meta_display_lookup_x_window (parent->display,
+                                                 parent->xtransient_for);
+        }
     }
+  else
+    transient_for = None;
+
+  if (transient_for == window->xtransient_for)
+    return;
+
+  if (meta_window_appears_focused (window) && window->xtransient_for != None)
+    meta_window_propagate_focus_appearance (window, FALSE);
+
+  old_transient_for = window->xtransient_for;
+  window->xtransient_for = transient_for;
 
   window->transient_parent_is_root_window =
     window->xtransient_for == window->screen->xroot;
@@ -1494,6 +1524,25 @@ reload_transient_for (MetaWindow    *window,
   /* may now be a dialog */
   meta_window_recalc_window_type (window);
 
+  if (!window->constructing)
+    {
+      /* If the window attaches, detaches, or changes attached
+       * parents, we need to destroy the MetaWindow and let a new one
+       * be created (which happens as a side effect of
+       * meta_window_unmanage()). The condition below is correct
+       * because we know window->xtransient_for has changed.
+       */
+      if (window->attached || meta_window_should_attach_to_parent (window))
+        {
+          guint32 timestamp;
+
+          window->xtransient_for = old_transient_for;
+          timestamp = meta_display_get_current_time_roundtrip (window->display);
+          meta_window_unmanage (window, timestamp);
+          return;
+        }
+    }
+
   /* update stacking constraints */
   if (!window->override_redirect)
     meta_stack_update_transient (window->screen->stack, window);
@@ -1509,6 +1558,38 @@ reload_transient_for (MetaWindow    *window,
 
   if (!window->constructing && !window->override_redirect)
     meta_window_queue (window, META_QUEUE_MOVE_RESIZE);
+
+  if (meta_window_appears_focused (window) && window->xtransient_for != None)
+    meta_window_propagate_focus_appearance (window, TRUE);
+}
+
+static void
+reload_gtk_theme_variant (MetaWindow    *window,
+                          MetaPropValue *value,
+                          gboolean       initial)
+{
+  char     *requested_variant = NULL;
+  char     *current_variant   = window->gtk_theme_variant;
+
+  if (value->type != META_PROP_VALUE_INVALID)
+    {
+      requested_variant = value->v.str;
+      meta_verbose ("Requested \"%s\" theme variant for window %s.\n",
+                    requested_variant, window->desc);
+    }
+
+  if (g_strcmp0 (requested_variant, current_variant))
+    {
+      g_free (current_variant);
+
+      if (requested_variant)
+        window->gtk_theme_variant = g_strdup (requested_variant);
+      else
+        window->gtk_theme_variant = NULL;
+
+      if (window->frame)
+        meta_ui_update_frame_style (window->screen->ui, window->frame->xwindow);
+    }
 }
 
 /**
@@ -1563,6 +1644,7 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
     { display->atom__NET_WM_STATE,     META_PROP_VALUE_ATOM_LIST, reload_net_wm_state,     TRUE,  FALSE },
     { display->atom__MOTIF_WM_HINTS,   META_PROP_VALUE_MOTIF_HINTS, reload_mwm_hints,      TRUE,  FALSE },
     { XA_WM_TRANSIENT_FOR,             META_PROP_VALUE_WINDOW,    reload_transient_for,    TRUE,  FALSE },
+    { display->atom__GTK_THEME_VARIANT, META_PROP_VALUE_UTF8,     reload_gtk_theme_variant, TRUE, FALSE },
     { display->atom__NET_WM_USER_TIME_WINDOW, META_PROP_VALUE_WINDOW, reload_net_wm_user_time_window, TRUE, FALSE },
     { display->atom_WM_STATE,          META_PROP_VALUE_INVALID,  NULL,                     FALSE, FALSE },
     { display->atom__NET_WM_ICON,      META_PROP_VALUE_INVALID,  reload_net_wm_icon,       FALSE, FALSE },
